@@ -1,24 +1,25 @@
-/*******************************************************************************
- * Copyright (c) 2011, 2015 Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 2011, 2021 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021 IBM Corporation. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Gordon Yorke - Initial development
- *
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Gordon Yorke - Initial development
+//
 
 package org.eclipse.persistence.internal.jpa.querydef;
 
 import java.lang.reflect.Constructor;
 import java.security.AccessController;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
+import java.util.*;
 
 import javax.persistence.Tuple;
 import javax.persistence.criteria.CriteriaQuery;
@@ -30,6 +31,7 @@ import javax.persistence.criteria.Selection;
 import javax.persistence.metamodel.Metamodel;
 import javax.persistence.metamodel.Type.PersistenceType;
 
+import org.eclipse.persistence.expressions.ExpressionBuilder;
 import org.eclipse.persistence.internal.helper.BasicTypeHelperImpl;
 import org.eclipse.persistence.internal.helper.ClassConstants;
 import org.eclipse.persistence.internal.jpa.metamodel.MetamodelImpl;
@@ -50,9 +52,9 @@ import org.eclipse.persistence.queries.ReportQuery;
  * <b>Description</b>: This is the container class for the components that
  * define a query.
  * <p>
- * 
+ *
  * @see javax.persistence.criteria CriteriaQuery
- * 
+ *
  * @author gyorke
  * @since EclipseLink 1.2
  */
@@ -61,7 +63,7 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
     protected SelectionImpl<?> selection;
     protected List<Order> orderBy;
 
-    protected List<FromImpl> joins;
+    protected Set<FromImpl> joins;
 
     public CriteriaQueryImpl(Metamodel metamodel, ResultType queryResult, Class result, CriteriaBuilderImpl queryBuilder) {
         super(metamodel, queryResult, queryBuilder, result);
@@ -70,19 +72,20 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
     /**
      * Specify the item that is to be returned in the query result. Replaces the
      * previously specified selection, if any.
-     * 
+     *
      * @param selection
      *            selection specifying the item that is to be returned in the
      *            query result
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> select(Selection<? extends T> selection) {
-        findRootAndParameters(selection);
         this.selection = (SelectionImpl) selection;
+        this.selection.findRootAndParameters(this);
         if (selection.isCompoundSelection()) {
             //bug 366386: validate that aliases are not reused
             if (this.selection.isCompoundSelection() && ((CompoundSelectionImpl)this.selection).getDuplicateAliasNames() != null) {
-                throw new IllegalArgumentException(ExceptionLocalization.buildMessage("jpa_criteriaapi_alias_reused", 
+                throw new IllegalArgumentException(ExceptionLocalization.buildMessage("jpa_criteriaapi_alias_reused",
                         new Object[] { ((CompoundSelectionImpl)this.selection).getDuplicateAliasNames() }));
             }
             if (selection.getJavaType().equals(Tuple.class)) {
@@ -115,49 +118,50 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
     /**
      * Specify the items that are to be returned in the query result. Replaces
      * the previously specified selection(s), if any.
-     * 
+     *
      * The type of the result of the query execution depends on the
      * specification of the criteria query object as well as the arguments to
      * the multiselect method as follows:
-     * 
-     * If the type of the criteria query is CriteriaQuery<Tuple>, a Tuple object
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;Tuple&gt;, a Tuple object
      * corresponding to the arguments of the multiselect method will be
      * instantiated and returned for each row that results from the query
      * execution.
-     * 
-     * If the type of the criteria query is CriteriaQuery<X> for some
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;X&gt; for some
      * user-defined class X, then the arguments to the multiselect method will
      * be passed to the X constructor and an instance of type X will be returned
      * for each row. The IllegalStateException will be thrown if a constructor
      * for the given argument types does not exist.
-     * 
-     * If the type of the criteria query is CriteriaQuery<X[]> for some class X,
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;X[]&gt; for some class X,
      * an instance of type X[] will be returned for each row. The elements of
      * the array will correspond to the arguments of the multiselect method. The
      * IllegalStateException will be thrown if the arguments to the multiselect
      * method are not of type X.
-     * 
-     * If the type of the criteria query is CriteriaQuery<Object>, and only a
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;Object&gt;, and only a
      * single argument is passed to the multiselect method, an instance of type
      * Object will be returned for each row.
-     * 
-     * If the type of the criteria query is CriteriaQuery<Object>, and more than
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;Object&gt;, and more than
      * one argument is passed to the multiselect method, an instance of type
      * Object[] will be instantiated and returned for each row. The elements of
      * the array will correspond to the arguments to the multiselect method.
-     * 
+     *
      * @param selections
      *            expressions specifying the items that are to be returned in
      *            the query result
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> multiselect(Selection<?>... selections) {
         if (selections == null || selections.length == 0) {
             this.selection = null;
             return this;
         }
         for (Selection select : selections) {
-            findRootAndParameters(select);
+            ((SelectionImpl)select).findRootAndParameters(this);
         }
         if (this.queryResult == ResultType.CONSTRUCTOR) {
             populateAndSetConstructorSelection(null, this.queryType, selections);
@@ -170,7 +174,7 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
                 } catch(IllegalArgumentException constructorDoesNotExist){
                     this.queryResult = ResultType.PARTIAL;
                     this.selection = new CompoundSelectionImpl(this.queryType, selections);
-                } 
+                }
             }
         } else if (this.queryResult.equals(ResultType.TUPLE)) {
             this.selection = new CompoundSelectionImpl(this.queryType, selections);
@@ -188,7 +192,7 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
         }
         //bug 366386: validate that aliases are not reused
         if (this.selection.isCompoundSelection() && ((CompoundSelectionImpl)this.selection).getDuplicateAliasNames() != null) {
-            throw new IllegalArgumentException(ExceptionLocalization.buildMessage("jpa_criteriaapi_alias_reused", 
+            throw new IllegalArgumentException(ExceptionLocalization.buildMessage("jpa_criteriaapi_alias_reused",
                     new Object[] { ((CompoundSelectionImpl)this.selection).getDuplicateAliasNames() }));
         }
         return this;
@@ -197,44 +201,45 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
     /**
      * Specify the items that are to be returned in the query result. Replaces
      * the previously specified selection(s), if any.
-     * 
+     *
      * The type of the result of the query execution depends on the
      * specification of the criteria query object as well as the arguments to
      * the multiselect method as follows:
-     * 
-     * If the type of the criteria query is CriteriaQuery<Tuple>, a Tuple object
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;Tuple&gt;, a Tuple object
      * corresponding to the items in the selection list passed to the
      * multiselect method will be instantiated and returned for each row that
      * results from the query execution.
-     * 
-     * If the type of the criteria query is CriteriaQuery<X> for some
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;X&gt; for some
      * user-defined class X, then the items in the selection list passed to the
      * multiselect method will be passed to the X constructor and an instance of
      * type X will be returned for each row. The IllegalStateException will be
      * thrown if a constructor for the given argument types does not exist.
-     * 
-     * If the type of the criteria query is CriteriaQuery<X[]> for some class X,
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;X[]&gt; for some class X,
      * an instance of type X[] will be returned for each row. The elements of
      * the array will correspond to the items in the selection list passed to
      * the multiselect method. The IllegalStateException will be thrown if the
      * elements in the selection list passed to the multiselect method are not
      * of type X.
-     * 
-     * If the type of the criteria query is CriteriaQuery<Object>, and the
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;Object&gt;, and the
      * selection list passed to the multiselect method contains only a single
      * item, an instance of type Object will be returned for each row.
-     * 
-     * If the type of the criteria query is CriteriaQuery<Object>, and the
+     *
+     * If the type of the criteria query is CriteriaQuery&lt;Object&gt;, and the
      * selection list passed to the multiselect method contains more than one
      * item, an instance of type Object[] will be instantiated and returned for
      * each row. The elements of the array will correspond to the items in the
      * selection list passed to the multiselect method.
-     * 
+     *
      * @param selectionList
      *            list of expressions specifying the items that to be are
      *            returned in the query result
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> multiselect(List<Selection<?>> selectionList) {
         if (selectionList == null) {
             this.selection = null;
@@ -249,11 +254,12 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * boolean expression. Replaces the previously added restriction(s), if any.
      * This method only overrides the return type of the corresponding
      * AbstractQuery method.
-     * 
+     *
      * @param restriction
      *            a simple or compound boolean expression
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> where(Expression<Boolean> restriction) {
         return (CriteriaQuery<T>) super.where(restriction);
     }
@@ -265,11 +271,12 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * specified, any previously added restrictions are simply removed. This
      * method only overrides the return type of the corresponding AbstractQuery
      * method.
-     * 
+     *
      * @param restrictions
      *            zero or more restriction predicates
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> where(Predicate... restrictions) {
         return (CriteriaQuery<T>) super.where(restrictions);
     }
@@ -280,11 +287,12 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * no grouping expressions are specified, any previously added grouping
      * expressions are simply removed. This method only overrides the return
      * type of the corresponding AbstractQuery method.
-     * 
+     *
      * @param grouping
      *            zero or more grouping expressions
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> groupBy(Expression<?>... grouping) {
         super.groupBy(grouping);
         return this;
@@ -296,11 +304,12 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * no grouping expressions are specified, any previously added grouping
      * expressions are simply removed. This method only overrides the return
      * type of the corresponding AbstractQuery method.
-     * 
+     *
      * @param grouping
      *            list of zero or more grouping expressions
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> groupBy(List<Expression<?>> grouping) {
         super.groupBy(grouping);
         return this;
@@ -310,11 +319,12 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * Specify a restriction over the groups of the query. Replaces the previous
      * having restriction(s), if any. This method only overrides the return type
      * of the corresponding AbstractQuery method.
-     * 
+     *
      * @param restriction
      *            a simple or compound boolean expression
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> having(Expression<Boolean> restriction) {
         super.having(restriction);
         return this;
@@ -327,11 +337,12 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * specified, any previously added restrictions are simply removed. This
      * method only overrides the return type of the corresponding AbstractQuery
      * method.
-     * 
+     *
      * @param restrictions
      *            zero or more restriction predicates
      * @return the modified query
      */
+    @Override
     public CriteriaQuery<T> having(Predicate... restrictions) {
         super.having(restrictions);
         return this;
@@ -344,11 +355,12 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * simply removed, and results will be returned in no particular order. The
      * left-to-right sequence of the ordering expressions determines the
      * precedence, whereby the leftmost has highest precedence.
-     * 
+     *
      * @param o
      *            zero or more ordering expressions
      * @return the modified query.
      */
+    @Override
     public CriteriaQuery<T> orderBy(Order... o) {
         this.orderBy = new ArrayList();
         for (Order order : o) {
@@ -364,24 +376,25 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * simply removed, and results will be returned in no particular order. The
      * order of the ordering expressions in the list determines the precedence,
      * whereby the first element in the list has highest precedence.
-     * 
+     *
      * @param o
      *            list of zero or more ordering expressions
      * @return the modified query.
      */
+    @Override
     public CriteriaQuery<T> orderBy(List<Order> o) {
         this.orderBy = o;
         return this;
     }
 
     /**
-     * This method will set this queryImpl's selection to a ConstructorSelectionImpl, creating a new 
-     * instance or populating the one passed in as necessary.    
-     * Throws IllegalArgumentException if a constructor taking arguments represented 
-     * by the selections array doesn't exist for the given class.  
-     * 
+     * This method will set this queryImpl's selection to a ConstructorSelectionImpl, creating a new
+     * instance or populating the one passed in as necessary.
+     * Throws IllegalArgumentException if a constructor taking arguments represented
+     * by the selections array doesn't exist for the given class.
+     *
      * Also sets the query result to ResultType.CONSTRUCTOR
-     * 
+     *
      * @param class1
      * @param selections
      * @throws IllegalArgumentException
@@ -390,6 +403,11 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
         Class[] constructorArgs = new Class[selections.length];
         int count = 0;
         for (Selection select : selections) {
+            if(select instanceof ConstructorSelectionImpl) {
+                ConstructorSelectionImpl constructorSelect = (ConstructorSelectionImpl)select;
+                Selection[] selectArray = constructorSelect.getCompoundSelectionItems().toArray(new Selection[constructorSelect.getCompoundSelectionItems().size()]);
+                populateAndSetConstructorSelection(constructorSelect, constructorSelect.getJavaType(), selectArray);
+            }
             constructorArgs[count++] = select.getJavaType();
         }
         Constructor constructor = null;
@@ -420,25 +438,28 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
      * duplicates to be retained. If distinct has not been specified, duplicate
      * results must be retained. This method only overrides the return type of
      * the corresponding AbstractQuery method.
-     * 
+     *
      * @param distinct
      *            boolean value specifying whether duplicate results must be
      *            eliminated from the query result or whether they must be
      *            retained
      * @return the modified query.
      */
+    @Override
     public CriteriaQuery<T> distinct(boolean distinct) {
         super.distinct(distinct);
         return this;
     }
 
+    @Override
     public void addJoin(FromImpl from) {
         if (this.joins == null) {
-            this.joins = new ArrayList<FromImpl>();
+            this.joins = new LinkedHashSet<FromImpl>();
         }
         this.joins.add(from);
     }
 
+    @Override
     protected DatabaseQuery getDatabaseQuery() {
         ObjectLevelReadQuery query = null;
         if (this.selection == null || !this.selection.isCompoundSelection()) {
@@ -451,9 +472,10 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
 
     /**
      * Return the ordering expressions in order of precedence.
-     * 
+     *
      * @return the list of ordering expressions
      */
+    @Override
     public List<Order> getOrderList() {
         return this.orderBy;
     }
@@ -461,9 +483,10 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
     /**
      * Return the selection item of the query. This will correspond to the query
      * type.
-     * 
+     *
      * @return the selection item of the query
      */
+    @Override
     public Selection<T> getSelection() {
         return (Selection<T>) this.selection;
     }
@@ -527,14 +550,41 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
                     }
                 }
             }
-            if (this.where != null && ((InternalSelection) this.where).getCurrentNode() != null  && 
-                    ((InternalSelection) this.where).getCurrentNode().getBuilder().getQueryClass() != null) {
-                reportQuery.setReferenceClass(((InternalSelection) this.where).getCurrentNode().getBuilder().getQueryClass());
-                reportQuery.setExpressionBuilder(((InternalSelection) this.where).getCurrentNode().getBuilder());
-            } else {
-                reportQuery.setExpressionBuilder(((InternalSelection) this.selection.getCompoundSelectionItems().get(0)).getCurrentNode().getBuilder());
-                reportQuery.setReferenceClass(((InternalSelection) this.selection.getCompoundSelectionItems().get(0)).getCurrentNode().getBuilder().getQueryClass());
+
+            ExpressionBuilder builder = null;
+            Class<?> queryClazz = null;
+            // First check the WHERE clause
+            if(this.where != null && ((InternalSelection) this.where).getCurrentNode() != null) {
+                builder = ((InternalSelection) this.where).getCurrentNode().getBuilder();
+                queryClazz = builder.getQueryClass();
             }
+            // Check all the SELECTION items next
+            if(queryClazz == null && this.selection != null) {
+                for(Selection<?> s : this.selection.getCompoundSelectionItems()) {
+                    if(((InternalSelection) s).getCurrentNode() != null ) {
+                        builder = ((InternalSelection) s).getCurrentNode().getBuilder();
+                        queryClazz = builder.getQueryClass();
+                        if(queryClazz != null) {
+                            break;
+                        }
+                    }
+                }
+            }
+            // Fallback on the root
+            if(queryClazz == null && this.roots != null) {
+                for(Root<?> r : this.roots) {
+                    if(((RootImpl<?>) r).getCurrentNode() != null ) {
+                        builder = ((RootImpl<?>) r).getCurrentNode().getBuilder();
+                        queryClazz = builder.getQueryClass();
+                        if(queryClazz != null) {
+                            break;
+                        }
+                    }
+                }
+            }
+            reportQuery.setExpressionBuilder(builder);
+            reportQuery.setReferenceClass(queryClazz);
+
             query = reportQuery;
             if (this.groupBy != null && !this.groupBy.isEmpty()) {
                 for (Expression<?> exp : this.groupBy) {
@@ -636,10 +686,10 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
                     //selecting size not all databases support subselect in select clause so convert to count/groupby
                     PathImpl collectionExpression = (PathImpl) ((FunctionExpressionImpl)this.selection).getChildExpressions().get(0);
                     ExpressionImpl fromExpression = (ExpressionImpl) collectionExpression.getParentPath();
-                    reportQuery.addAttribute(this.selection.getAlias(), collectionExpression.getCurrentNode().count(), ClassConstants.INTEGER); 
+                    reportQuery.addAttribute(this.selection.getAlias(), collectionExpression.getCurrentNode().count(), ClassConstants.INTEGER);
                     reportQuery.addGrouping(fromExpression.getCurrentNode());
                 }else{
-                    reportQuery.addAttribute(this.selection.getAlias(), ((FunctionExpressionImpl)this.selection).getCurrentNode(), this.selection.getJavaType()); 
+                    reportQuery.addAttribute(this.selection.getAlias(), ((FunctionExpressionImpl)this.selection).getCurrentNode(), this.selection.getJavaType());
 
                 }}else{
                 if (((InternalSelection) selection).isFrom()) {
@@ -687,6 +737,7 @@ public class CriteriaQueryImpl<T> extends AbstractQueryImpl<T> implements Criter
     /**
      * Translates from the criteria query to a EclipseLink Database Query.
      */
+    @Override
     public DatabaseQuery translate() {
         ObjectLevelReadQuery query = (ObjectLevelReadQuery)super.translate();
 
