@@ -1,34 +1,53 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2017 Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 1998, 2020 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2020 IBM Corporation. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- *     07/13/2012-2.5 Guy Pelletier 
- *       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls
- *     08/24/2012-2.5 Guy Pelletier 
- *       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls
- *     01/31/2017-2.6 Will Dazey
- *       - 511426: Adding cloning support
- ******************************************************************************/  
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     07/13/2012-2.5 Guy Pelletier
+//       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls
+//     08/24/2012-2.5 Guy Pelletier
+//       - 350487: JPA 2.1 Specification defined support for Stored Procedure Calls
+//     01/31/2017-2.6 Will Dazey
+//       - 511426: Adding cloning support
+//     04/11/2018 - Will Dazey
+//       - 533148 : Add the eclipselink.jpa.sql-call-deferral property
 package org.eclipse.persistence.internal.queries;
 
-import java.util.*;
-import org.eclipse.persistence.internal.helper.*;
-import org.eclipse.persistence.internal.databaseaccess.Accessor;
-import org.eclipse.persistence.internal.databaseaccess.DatasourceCall;
-import org.eclipse.persistence.internal.databaseaccess.DatabaseCall;
+import java.util.Collection;
+import java.util.Enumeration;
+import java.util.List;
+import java.util.Vector;
+
 import org.eclipse.persistence.descriptors.ClassDescriptor;
-import org.eclipse.persistence.exceptions.*;
-import org.eclipse.persistence.queries.*;
+import org.eclipse.persistence.exceptions.DatabaseException;
+import org.eclipse.persistence.exceptions.QueryException;
+import org.eclipse.persistence.internal.databaseaccess.Accessor;
+import org.eclipse.persistence.internal.databaseaccess.DatabaseCall;
+import org.eclipse.persistence.internal.databaseaccess.DatasourceCall;
+import org.eclipse.persistence.internal.helper.DatabaseField;
+import org.eclipse.persistence.internal.helper.DatabaseTable;
+import org.eclipse.persistence.internal.helper.Helper;
+import org.eclipse.persistence.internal.helper.NonSynchronizedVector;
 import org.eclipse.persistence.internal.sessions.AbstractRecord;
 import org.eclipse.persistence.internal.sessions.AbstractSession;
+import org.eclipse.persistence.mappings.DatabaseMapping;
 import org.eclipse.persistence.mappings.DatabaseMapping.WriteType;
+import org.eclipse.persistence.queries.ConstructorReportItem;
+import org.eclipse.persistence.queries.DatabaseQuery;
+import org.eclipse.persistence.queries.DeleteAllQuery;
+import org.eclipse.persistence.queries.ReportQuery;
+import org.eclipse.persistence.queries.UpdateAllQuery;
+import org.eclipse.persistence.queries.WriteObjectQuery;
 
 /**
  * <p><b>Purpose</b>:
@@ -48,7 +67,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
 
     public DatasourceCallQueryMechanism() {
     }
-    
+
     /**
      * Initialize the state of the query
      * @param query - owner of mechanism
@@ -102,7 +121,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
             throw QueryException.mustUseCursorStreamPolicy();
         }
     }
-    
+
     /**
      * Read all rows from the database, return ResultSet
      * @exception  DatabaseException - an error has occurred on the database
@@ -110,7 +129,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
     public DatabaseCall selectResultSet() throws DatabaseException {
         try {
             // For CR 2923 must move to session we will execute call on now
-            // so correct DatasourcePlatform used by translate.        
+            // so correct DatasourcePlatform used by translate.
             AbstractSession sessionToUse = this.query.getExecutionSession();
             DatabaseCall clonedCall = (DatabaseCall)this.call.clone();
             clonedCall.setQuery(this.query);
@@ -133,7 +152,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         } else {
             if (hasMultipleCalls()) {
                 Integer returnedRowCount = null;
-                
+
                 // Deletion must occur in reverse order.
                 for (int index = getCalls().size() - 1; index >= 0; index--) {
                     DatasourceCall databseCall = (DatasourceCall)getCalls().elementAt(index);
@@ -155,7 +174,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
     public Integer deleteAllUsingTempTables() throws DatabaseException {
         DatabaseException ex = null;
         Integer returnedRowCount = null;
-        
+
         // Deletion must occur in reverse order.
 
         // first call - crete temp table.
@@ -165,19 +184,17 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
             executeCall(databseCall);
         } catch (DatabaseException databaseEx) {
             // ignore
-        }                
+        }
 
         // second call - populate temp table.
         // if that fails save the exception and untill cleanup
-        if(ex == null) {
-            try {
-                DatasourceCall databseCall = (DatasourceCall)getCalls().elementAt(getCalls().size() - 2);
-                executeCall(databseCall);
-            } catch (DatabaseException databaseEx) {
-                ex = databaseEx;
-            }
+        try {
+            DatasourceCall databseCall = (DatasourceCall)getCalls().elementAt(getCalls().size() - 2);
+            executeCall(databseCall);
+        } catch (DatabaseException databaseEx) {
+            ex = databaseEx;
         }
-        
+
         // third (a call per table) - delete from original tables calls.
         // if that fails save the exception untill cleanup
         for (int index = getCalls().size() - 3; index >= 1 && ex == null; index--) {
@@ -202,7 +219,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         if(ex != null) {
             throw ex;
         }
-        
+
         return returnedRowCount;
     }
 
@@ -237,7 +254,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
     public Object execute() throws DatabaseException {
         return executeCall();
     }
-    
+
     /**
      * Execute the call.  It is assumed the call has been fully prepared.
      * @exception  DatabaseException - an error has occurred on the database.
@@ -252,7 +269,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
      */
     protected Object executeCall(DatasourceCall databaseCall) throws DatabaseException {
         // For CR 2923 must move to session we will execute call on now
-        // so correct DatasourcePlatform used by translate.        
+        // so correct DatasourcePlatform used by translate.
         AbstractSession sessionToUse = this.query.getExecutionSession();
         DatasourceCall clonedCall = (DatasourceCall)databaseCall.clone();
         clonedCall.setQuery(this.query);
@@ -356,8 +373,8 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
             shouldAcquireValueAfterInsert = descriptor.getSequence().shouldAcquireValueAfterInsert();
         }
         Collection returnFields = null;
-        if (descriptor.hasReturningPolicy()) {
-            returnFields = descriptor.getReturningPolicy().getFieldsToMergeInsert();
+        if (descriptor.getReturnFieldsToMergeInsert() != null) {
+            returnFields = descriptor.getReturnFieldsToMergeInsert();
         }
 
         // Check to see if sequence number should be retrieved after insert
@@ -374,7 +391,8 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
             for (int index = 0; index < size; index++) {
                 DatasourceCall databseCall = (DatasourceCall)this.calls.get(index);
                 if ((index > 0) && isExpressionQueryMechanism()
-                        && this.query.shouldCascadeOnlyDependentParts() && !descriptor.hasMultipleTableConstraintDependecy()) {
+                        && this.query.shouldCascadeOnlyDependentParts() && !descriptor.hasMultipleTableConstraintDependecy()
+                        && this.query.getSession().getProject().allowSQLDeferral()) {
                     DatabaseTable table = descriptor.getMultipleTableInsertOrder().get(index);
                     this.query.getSession().getCommitManager().addDeferredCall(table, databseCall, this);
                 } else {
@@ -412,7 +430,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         //Oracle thin driver handles LOB differently. During the insert, empty lob would be
         //insert first, and then the LOb locator is retrieved and LOB data are written through
         //the locator.
-        // 
+        //
         // Bug 2804663 - LOBValueWriter is no longer a singleton, so we execute any deferred
         // select calls through the DatabaseAccessor which holds the writer instance
         AbstractSession executionSession = this.query.getExecutionSession();
@@ -439,7 +457,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
             updateObjectAndRowWithReturnRow(returnFields, false);
         }
     }
-    
+
     /**
      * Return true if this is a call query mechanism
      */
@@ -478,7 +496,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
             getCall().prepare(executionSession);
         }
     }
-    
+
     /**
      * Pre-build configure the call.
      */
@@ -548,14 +566,14 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
     /**
      * Pre-build configure the call. This method assumes the query was built
      * using a stored procedure query which is a single call.
-     * 
-     * The return type on the call will already be set and 
+     *
+     * The return type on the call will already be set and
      */
     public void prepareExecute() {
         getCall().setExecuteUpdate();
         prepareCall();
     }
-    
+
     /**
      * Pre-build configure the call.
      */
@@ -599,24 +617,40 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         //calculate indexes after normalize to insure expressions are set up correctly
         //take into account any field expressions added to the ReportQuery
         ReportQuery query = (ReportQuery)getQuery();
-        int itemOffset = query.getQueryExpressions().size();
-        for (Iterator items = query.getItems().iterator(); items.hasNext();) {
-            ReportItem item = (ReportItem) items.next();
-            item.setResultIndex(itemOffset);
-            if (item.getAttributeExpression() != null) {
-                if (item.hasJoining()){
-                    itemOffset = item.getJoinedAttributeManager().computeJoiningMappingIndexes(true, getSession(), itemOffset);
-                } else {
-                    if (item.getDescriptor() != null) {
-                        itemOffset += item.getDescriptor().getAllSelectionFields(query).size();
+        computeAndSetItemOffset(query, query.getItems(), query.getQueryExpressions().size());
+    }
+
+    /**
+     * calculate indexes for given items, given the current Offset
+     */
+    protected int computeAndSetItemOffset(ReportQuery query, List<ReportItem> items, int itemOffset) {
+        for(ReportItem item : items) {
+            if (item.isConstructorItem()) {
+                List<ReportItem> reportItems = ((ConstructorReportItem) item).getReportItems();
+                itemOffset = computeAndSetItemOffset(query, reportItems, itemOffset);
+            } else {
+                //Don't set the offset on the ConstructorItem
+                item.setResultIndex(itemOffset);
+                if (item.getAttributeExpression() != null) {
+                    if (item.hasJoining()){
+                        itemOffset = item.getJoinedAttributeManager().computeJoiningMappingIndexes(true, getSession(), itemOffset);
                     } else {
-                        ++itemOffset; //only a single attribute can be selected
+                        if (item.getDescriptor() != null) {
+                            itemOffset += item.getDescriptor().getAllSelectionFields(query).size();
+                        } else {
+                            if (item.getMapping() != null && item.getMapping().isAggregateObjectMapping()) {
+                                itemOffset += item.getMapping().getFields().size(); // Aggregate object may consist out of 1..n fields
+                            } else {
+                                ++itemOffset; //only a single attribute can be selected
+                            }
+                        }
                     }
                 }
             }
-        }        
+        }
+        return itemOffset;
     }
-    
+
     /**
      * Pre-build configure the call.
      */
@@ -779,10 +813,10 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
      * @return the row count.
      */
     public Integer updateObject() throws DatabaseException {
-        Collection returnFields = null;
         ClassDescriptor descriptor = getDescriptor();
-        if (descriptor.hasReturningPolicy()) {
-            returnFields = descriptor.getReturningPolicy().getFieldsToMergeUpdate();
+        Collection returnFields = null;
+        if (descriptor.getReturnFieldsToMergeUpdate() != null) {
+            returnFields = descriptor.getReturnFieldsToMergeUpdate();
         }
         Integer returnedRowCount = null;
         if (hasMultipleCalls()) {
@@ -790,7 +824,8 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
             for (int index = 0; index < size; index++) {
                 DatasourceCall databseCall = (DatasourceCall)this.calls.get(index);
                 if ((index > 0) && isExpressionQueryMechanism()
-                        && this.query.shouldCascadeOnlyDependentParts() && !descriptor.hasMultipleTableConstraintDependecy()) {
+                        && this.query.shouldCascadeOnlyDependentParts() && !descriptor.hasMultipleTableConstraintDependecy()
+                        && this.query.getSession().getProject().allowSQLDeferral()) {
                     DatabaseTable table = descriptor.getMultipleTableInsertOrder().get(index);
                     this.query.getSession().getCommitManager().addDeferredCall(table, databseCall, this);
                 } else {
@@ -828,7 +863,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         //Oracle thin driver handles LOB differently. During the insert, empty lob would be
         //insert first, and then the LOb locator is retrieved and LOB data are written through
         //the locator.
-        // 
+        //
         // Bug 2804663 - LOBValueWriter is no longer a singleton, so we execute any deferred
         // select calls through the DatabaseAccessor which holds the writer instance
         //
@@ -852,7 +887,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         } else {
             Integer rowCount = executeNoSelectCall();
             if(((UpdateAllQuery)this.query).isPreparedUsingTempStorage()) {
-                // the query was prepared using Oracle anonymous block 
+                // the query was prepared using Oracle anonymous block
                 AbstractRecord outputRow = (AbstractRecord)this.query.getProperty("output");
                 rowCount = (Integer)outputRow.get("ROW_COUNT");
             }
@@ -869,7 +904,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         int nTables = getCalls().size() / 4;
         DatabaseException ex = null;
         Integer returnedRowCount = null;
-        
+
         // first quarter - crete temp tables calls.
         // may fail in case global temp table already exists.
         for (int index = 0; index < nTables; index++) {
@@ -891,7 +926,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
                 ex = databaseEx;
             }
         }
-        
+
         // third quarter - update original tables calls.
         // if that fails save the exception and until cleanup
         for (int index = nTables*2; index < nTables*3 && ex == null; index++) {
@@ -905,7 +940,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
                 ex = databaseEx;
             }
         }
-        
+
         // last quarter - cleanup temp tables calls.
         // ignore exceptions here.
         for (int index = nTables*3; index < nTables*4; index++) {
@@ -920,7 +955,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         if(ex != null) {
             throw ex;
         }
-        
+
         return returnedRowCount;
     }
 
@@ -932,7 +967,7 @@ public class DatasourceCallQueryMechanism extends DatabaseQueryMechanism {
         writeQuery.setModifyRow(this.getDescriptor().getObjectBuilder().buildRow(writeQuery.getObject(), this.getSession(), WriteType.INSERT));
 
         // For CR 2923 must move to session we will execute call on now
-        // so correct DatasourcePlatform used by translate. 
+        // so correct DatasourcePlatform used by translate.
         AbstractSession sessionToUse = this.query.getExecutionSession();
 
         // yes - this is a bit ugly...
