@@ -1,65 +1,82 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2015 Oracle and/or its affiliates, IBM Corporation. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 1998, 2020 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020 IBM Corporation. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- *     14/05/2012-2.4 Guy Pelletier  
- *       - 376603: Provide for table per tenant support for multitenant applications
- *     22/05/2012-2.4 Guy Pelletier  
- *       - 380008: Multitenant persistence units with a dedicated emf should force tenant property specification up front.
- *     31/05/2012-2.4 Guy Pelletier  
- *       - 381196: Multitenant persistence units with a dedicated emf should allow for DDL generation.
- *     12/24/2012-2.5 Guy Pelletier 
- *       - 389090: JPA 2.1 DDL Generation Support
- *     01/11/2013-2.5 Guy Pelletier 
- *       - 389090: JPA 2.1 DDL Generation Support
- *     03/19/2015 - Rick Curtis
- *       - 462586 : Add national character support for z/OS.
- *     04/14/2015 - Will Dazey
- *       - 464641 : Fixed platform matching returning CNF.
- *     09/03/2015 - Will Dazey
- *       - 456067 : Added support for defining query timeout units
- *       ******************************************************************************/  
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
+//     14/05/2012-2.4 Guy Pelletier
+//       - 376603: Provide for table per tenant support for multitenant applications
+//     22/05/2012-2.4 Guy Pelletier
+//       - 380008: Multitenant persistence units with a dedicated emf should force tenant property specification up front.
+//     31/05/2012-2.4 Guy Pelletier
+//       - 381196: Multitenant persistence units with a dedicated emf should allow for DDL generation.
+//     12/24/2012-2.5 Guy Pelletier
+//       - 389090: JPA 2.1 DDL Generation Support
+//     01/11/2013-2.5 Guy Pelletier
+//       - 389090: JPA 2.1 DDL Generation Support
+//     03/19/2015 - Rick Curtis
+//       - 462586 : Add national character support for z/OS.
+//     04/14/2015 - Will Dazey
+//       - 464641 : Fixed platform matching returning CNF.
+//     09/03/2015 - Will Dazey
+//       - 456067 : Added support for defining query timeout units
+//     06/26/2018 - Will Dazey
+//       - 532160 : Add support for non-extension OracleXPlatform classes
 package org.eclipse.persistence.internal.sessions;
 
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
 import java.sql.SQLException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Date;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Vector;
 
 import org.eclipse.persistence.config.PersistenceUnitProperties;
 import org.eclipse.persistence.descriptors.ClassDescriptor;
 import org.eclipse.persistence.descriptors.partitioning.PartitioningPolicy;
-import org.eclipse.persistence.exceptions.*;
-import org.eclipse.persistence.internal.helper.DBPlatformHelper;
+import org.eclipse.persistence.exceptions.DatabaseException;
+import org.eclipse.persistence.exceptions.EclipseLinkException;
+import org.eclipse.persistence.exceptions.IntegrityException;
+import org.eclipse.persistence.exceptions.OptimisticLockException;
+import org.eclipse.persistence.exceptions.ValidationException;
 import org.eclipse.persistence.internal.databaseaccess.Platform;
+import org.eclipse.persistence.internal.helper.DBPlatformHelper;
 import org.eclipse.persistence.internal.sequencing.Sequencing;
-import org.eclipse.persistence.internal.sequencing.SequencingHome;
 import org.eclipse.persistence.internal.sequencing.SequencingFactory;
-import org.eclipse.persistence.sequencing.Sequence;
-import org.eclipse.persistence.sequencing.SequencingControl;
+import org.eclipse.persistence.internal.sequencing.SequencingHome;
 import org.eclipse.persistence.logging.SessionLog;
-import org.eclipse.persistence.sessions.Login;
-import org.eclipse.persistence.sessions.DatasourceLogin;
-import org.eclipse.persistence.sessions.SessionProfiler;
-import org.eclipse.persistence.tools.tuning.SessionTuner;
 import org.eclipse.persistence.platform.database.DatabasePlatform;
 import org.eclipse.persistence.platform.database.OraclePlatform;
 import org.eclipse.persistence.platform.database.events.DatabaseEventListener;
-import org.eclipse.persistence.platform.server.ServerPlatform;
 import org.eclipse.persistence.platform.server.NoServerPlatform;
+import org.eclipse.persistence.platform.server.ServerPlatform;
 import org.eclipse.persistence.platform.server.ServerPlatformBase;
 import org.eclipse.persistence.queries.AttributeGroup;
 import org.eclipse.persistence.queries.DatabaseQuery;
 import org.eclipse.persistence.queries.ObjectLevelReadQuery;
 import org.eclipse.persistence.queries.QueryResultsCachePolicy;
 import org.eclipse.persistence.queries.ReadQuery;
+import org.eclipse.persistence.sequencing.Sequence;
+import org.eclipse.persistence.sequencing.SequencingControl;
+import org.eclipse.persistence.sessions.DatasourceLogin;
+import org.eclipse.persistence.sessions.Login;
+import org.eclipse.persistence.sessions.SessionProfiler;
+import org.eclipse.persistence.tools.tuning.SessionTuner;
 
 /**
  * Implementation of org.eclipse.persistence.sessions.DatabaseSession
@@ -101,7 +118,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * Used to store the server platform that handles server-specific functionality for Oc4j, WLS,  etc.
      */
     protected ServerPlatform serverPlatform;
-    
+
     /**
      * Stores the tuner used to tune the configuration of this session.
      */
@@ -118,9 +135,9 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * Indicate if this session is logged in.
      */
 
-    //Bug#3440544 Used to stop the attempt to login more than once. 
+    //Bug#3440544 Used to stop the attempt to login more than once.
     protected volatile boolean isLoggedIn;
-    
+
     /**
      * INTERNAL:
      * Set the SequencingHome object used by the session.
@@ -149,25 +166,27 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
     public boolean isLoggedIn() {
         return isLoggedIn;
     }
-    
+
     /**
      * Return the database event listener, this allows database events to invalidate the cache.
      */
+    @Override
     public DatabaseEventListener getDatabaseEventListener() {
         return databaseEventListener;
     }
-    
+
     /**
      * PUBLIC:
      * Set the database event listener, this allows database events to invalidate the cache.
      */
+    @Override
     public void setDatabaseEventListener(DatabaseEventListener databaseEventListener) {
         this.databaseEventListener = databaseEventListener;
     }
-    
+
     /**
      * INTERNAL:
-     * Issue any pre connect and post connect without an actual connection to 
+     * Issue any pre connect and post connect without an actual connection to
      * the database. Descriptors are initialized in postConnectDatasource and
      * are used in DDL generation. This will look to set the schema platform
      * via the JPA 2.1 properties or through a detection on the connection
@@ -178,42 +197,36 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         setOrDetectDatasource(false);
         postConnectDatasource();
     }
-    
+
     /**
      * INTERNAL:
-     * Will set the platform from specified schema generation properties or 
+     * Will set the platform from specified schema generation properties or
      * by detecting it through the connection (if one is available).
      * Any connection that is open for detection is closed before this method
      * returns.
-     * 
+     *
      * @param throwException - set to true if the caller cares to throw exceptions, false to swallow them.
      */
     protected void setOrDetectDatasource(boolean throwException) {
-        String vendorNameAndVersion = null;
+        String vendorName = null;
+        String minorVersion = null;
+        String majorVersion = null;
         String driverName = null;
 
         // Try to set the platform from JPA 2.1 schema properties first before
         // attempting a detection.
         if (getProperties().containsKey(PersistenceUnitProperties.SCHEMA_DATABASE_PRODUCT_NAME)) {
-            vendorNameAndVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_PRODUCT_NAME);
-            
-            String majorVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_MAJOR_VERSION);
-            if (majorVersion != null) {
-                vendorNameAndVersion += majorVersion;
-            }
-            
-            // The minorVersion is not currently used in platform matching, but
-            // shouldn't change matching when added
-            String minorVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_MINOR_VERSION);
-            if (minorVersion != null) {
-                vendorNameAndVersion += minorVersion;
-            }
+            vendorName = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_PRODUCT_NAME);
+            minorVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_MINOR_VERSION);
+            majorVersion = (String) getProperties().get(PersistenceUnitProperties.SCHEMA_DATABASE_MAJOR_VERSION);
         } else {
             Connection conn = null;
             try {
                 conn = (Connection) getReadLogin().connectToDatasource(null, this);
                 DatabaseMetaData dmd = conn.getMetaData();
-                vendorNameAndVersion = dmd.getDatabaseProductName() + dmd.getDatabaseMajorVersion() + dmd.getDatabaseProductVersion();
+                vendorName = dmd.getDatabaseProductName();
+                minorVersion = dmd.getDatabaseProductVersion();
+                majorVersion = Integer.toString(dmd.getDatabaseMajorVersion());
                 driverName = conn.getMetaData().getDriverName();
             } catch (SQLException ex) {
                 if (throwException) {
@@ -249,29 +262,36 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             // null out the cached platform because the platform on the login
             // will be changed by the following line of code
             this.platform = null;
-            platformName = DBPlatformHelper.getDBPlatform(vendorNameAndVersion, getSessionLog());
+            platformName = DBPlatformHelper.getDBPlatform(vendorName, minorVersion, majorVersion, getSessionLog());
             getLogin().setPlatformClassName(platformName);
-            if (driverName != null) {
-                getLogin().getPlatform().setDriverName(driverName);
-            }
         } catch (EclipseLinkException classNotFound) {
-            if (platformName.indexOf("Oracle") != -1) {
-                // If we are running against Oracle, it is possible that we are
-                // running in an environment where
-                // the OracleXPlatform class can not be loaded. Try using
-                // OraclePlatform class before giving up
-                getLogin().setPlatformClassName(OraclePlatform.class.getName());
+            if (platformName != null && platformName.indexOf("Oracle") != -1) {
+                try {
+                    // If we are running against Oracle, it is possible that we are
+                    // running in an environment where the extension OracleXPlatform classes can 
+                    // not be loaded. Try using the core OracleXPlatform classes
+                    platformName = DBPlatformHelper.getDBPlatform("core."+ vendorName, minorVersion, majorVersion, getSessionLog());
+                    getLogin().setPlatformClassName(platformName);
+                } catch (EclipseLinkException oracleClassNotFound) {
+                    // If we still cannot classload a matching OracleXPlatform class, 
+                    // fallback on the base OraclePlatform class
+                    getLogin().setPlatformClassName(OraclePlatform.class.getName());
+                }
             } else {
                 throw classNotFound;
             }
         }
+        if (driverName != null) {
+            getLogin().getPlatform().setDriverName(driverName);
+        }
     }
-    
+
     /**
      * PUBLIC:
      * Return  SequencingControl which used for sequencing setup and
      * customization including management of sequencing preallocation.
      */
+    @Override
     public SequencingControl getSequencingControl() {
         return getSequencingHome().getSequencingControl();
     }
@@ -280,6 +300,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * PUBLIC:
      * Return the Sequencing object used by the session.
      */
+    @Override
     public Sequencing getSequencing() {
         return getSequencingHome().getSequencing();
     }
@@ -316,6 +337,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * Called in the end of beforeCompletion of external transaction synchronization listener.
      * Close the managed sql connection corresponding to the external transaction.
      */
+    @Override
     public void releaseJTSConnection() {
         getAccessor().closeJTSConnection();
     }
@@ -370,7 +392,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * @see #addDescriptors(org.eclipse.persistence.sessions.Project)
      */
     @Override
-    public void addDescriptor(final ClassDescriptor descriptor) {
+    public void addDescriptor(ClassDescriptor descriptor) {
         // Reset cached data, as may be invalid later on.
         this.lastDescriptorAccessed = null;
         // Bug# 429760: Add descriptor to the session when session Map exists and is not the same as in the project.
@@ -386,6 +408,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * All persistent classes must have a descriptor registered for them with the session.
      * This method allows for a batch of descriptors to be added at once so that EclipseLink
      * can resolve the dependencies between the descriptors and perform initialization optimally.
+     * @param descriptors The descriptors to be added to the session and project.
      */
     @Override
     public void addDescriptors(final Collection descriptors) {
@@ -406,6 +429,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * This can be used to combine the descriptors from multiple projects into a single session.
      * This can be called after the session has been connected as long as there are no external dependencies.
      */
+    @Override
     public void addDescriptors(org.eclipse.persistence.sessions.Project project) {
         // Reset cached data, as may be invalid later on.
         this.lastDescriptorAccessed = null;
@@ -417,17 +441,18 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * PUBLIC:
      * Add the sequence to the session.
      * Allows to add a new sequence to the session even if the session is connected.
-     * If the session is connected then the sequence is added only 
+     * If the session is connected then the sequence is added only
      * if there is no sequence with the same name already in use.
-     * Call this method before addDescriptor(s) if need to add new descriptor 
+     * Call this method before addDescriptor(s) if need to add new descriptor
      * with a new non-default sequence to connected session.
      *
      * @see #addSequences(Collection)
      */
+    @Override
     public void addSequence(Sequence sequence) {
-        getProject().getLogin().getDatasourcePlatform().addSequence(sequence, this.getSequencingHome().isConnected());        
+        getProject().getLogin().getDatasourcePlatform().addSequence(sequence, this.getSequencingHome().isConnected());
     }
-    
+
     /**
      * INTERNAL:
      * Connect the session only.
@@ -462,6 +487,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      *
      * @see org.eclipse.persistence.platform.server.ServerPlatformBase
      */
+    @Override
     public ServerPlatform getServerPlatform() {
         return serverPlatform;
     }
@@ -485,6 +511,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      *
      * @see org.eclipse.persistence.platform.server.ServerPlatformBase
      */
+    @Override
     public void setServerPlatform(ServerPlatform newServerPlatform) {
         if (this.isLoggedIn) {
             throw ValidationException.serverPlatformIsReadOnlyAfterLogin(newServerPlatform.getClass().getName());
@@ -496,6 +523,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * INTERNAL:
      * Logout in case still connected.
      */
+    @Override
     protected void finalize() throws DatabaseException {
         if (isConnected()) {
             logout();
@@ -509,6 +537,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * NOTE: this must only be used for relational specific usage,
      * it will fail for non-relational datasources.
      */
+    @Override
     public DatabasePlatform getPlatform() {
         // PERF: Cache the platform.
         if (platform == null) {
@@ -526,6 +555,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * Return the database platform currently connected to.
      * The platform is used for database specific behavior.
      */
+    @Override
     public Platform getDatasourcePlatform() {
         // PERF: Cache the platform.
         if (platform == null) {
@@ -544,6 +574,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * for specified class.
      * The platform is used for database specific behavior.
      */
+    @Override
     public Platform getPlatform(Class domainClass) {
         // PERF: Cache the platform.
         if (platform == null) {
@@ -598,7 +629,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         initializeDescriptors((Map)((HashMap)getDescriptors()).clone(), true);
         // Initialize serializer
         if (this.serializer != null) {
-        	this.serializer.initialize(null, null, this);
+            this.serializer.initialize(null, null, this);
         }
         // Initialize partitioning policies.
         for (PartitioningPolicy policy : getProject().getPartitioningPolicies().values()) {
@@ -612,7 +643,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         // Process JPA named queries and add as session queries,
         // this must be done after descriptor init as requires to parse the JPQL.
         processJPAQueries();
-        
+
         // Configure default query cache for all named queries.
         QueryResultsCachePolicy defaultQueryCachePolicy = getProject().getDefaultQueryResultsCachePolicy();
         if (defaultQueryCachePolicy != null) {
@@ -659,7 +690,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * If shouldInitializeSequencing parameter is false then
      *   if sequencing has been already connected, then it stays connected:
      *     only the new sequences used by the passed descriptors are initialized;
-     *   otherwise, if sequencing has NOT been connected then it is connected 
+     *   otherwise, if sequencing has NOT been connected then it is connected
      *     (just like in shouldInitializeSequencing==true case);
      *   disconnected (if has been connected), then connected.
      */
@@ -672,7 +703,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         } else {
             addDescriptorsToSequencing(descriptors);
         }
-        
+
         try {
             // First initialize basic properties (things that do not depend on anything else)
             Iterator iterator = descriptors.iterator();
@@ -688,7 +719,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
                         // to be cloned and initialized per client session.
                         addTablePerTenantDescriptor(descriptor);
                     }
-                    
+
                     //check if inheritance is involved in aggregate relationship, and let the parent know the child descriptor
                     if (descriptor.isDescriptorTypeAggregate() && descriptor.isChildDescriptor()) {
                         descriptor.initializeAggregateInheritancePolicy(session);
@@ -741,6 +772,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * INTERNAL:
      * Return if this session is a database session.
      */
+    @Override
     public boolean isDatabaseSession() {
         return true;
     }
@@ -749,6 +781,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * PUBLIC:
      * Returns true if Protected Entities should be built within this session
      */
+    @Override
     public boolean isProtectedSession(){
         return false;
     }
@@ -760,7 +793,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
     protected Login getReadLogin(){
         return getDatasourceLogin();
     }
-    
+
     /**
      * PUBLIC:
      * Connect to the database using the predefined login.
@@ -785,12 +818,13 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      *
      * @see #login(Login)
      */
+    @Override
     public void login() throws DatabaseException {
         preConnectDatasource();
         connect();
         postConnectDatasource();
     }
-    
+
     /**
      * INTERNAL:
      * This method includes all of the code that is issued before the datasource
@@ -816,7 +850,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             //setup the external transaction controller
             getServerPlatform().initializeExternalTransactionController();
             log(SessionLog.INFO, null, "topLink_version", DatasourceLogin.getVersion());
-            if (getServerPlatform().getServerNameAndVersion() != null && 
+            if (getServerPlatform().getServerNameAndVersion() != null &&
                     !getServerPlatform().getServerNameAndVersion().equals(ServerPlatformBase.DEFAULT_SERVER_NAME_AND_VERSION)) {
                 log(SessionLog.INFO, null, "application_server_name_and_version", getServerPlatform().getServerNameAndVersion());
             }
@@ -832,34 +866,34 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
     protected void postConnectDatasource(){
         if (!hasBroker()) {
             initializeDescriptors();
-            
+
             //added to process ejbQL query strings
             if (getCommandManager() != null) {
                 getCommandManager().initialize();
             }
         }
-        
+
         // Once the descriptors are initialized we can check if there are
         // multitenant entities and if this session (emf) is shared or not. If
-        // not shared, all multitenant properties must be available and set by 
-        // the user at this point for us to validate (meaning they must be set 
+        // not shared, all multitenant properties must be available and set by
+        // the user at this point for us to validate (meaning they must be set
         // in a persitence.xml or passed into the create EMF call).
         if (getProperties().containsKey(PersistenceUnitProperties.MULTITENANT_SHARED_EMF)) {
             String value = (String) getProperties().get(PersistenceUnitProperties.MULTITENANT_SHARED_EMF);
-            if (! new Boolean(value)) {
+            if (!Boolean.valueOf(value)) {
                 for (String property : getMultitenantContextProperties()) {
                     if (! getProperties().containsKey(property)) {
                         throw ValidationException.multitenantContextPropertyForNonSharedEMFNotSpecified(property);
                     }
                 }
-                
+
                 // Once the properties are validated we can allow ddl generation to happen (if needed).
                 project.setAllowTablePerMultitenantDDLGeneration(true);
             }
         }
-        
-        log(SessionLog.INFO, SessionLog.CONNECTION, "login_successful", this.getName());
-        // postLogin event should not be risen before descriptors have been initialized 
+
+        log(SessionLog.FINE, SessionLog.CONNECTION, "login_successful", this.getName());
+        // postLogin event should not be risen before descriptors have been initialized
         if (!hasBroker()) {
             postLogin();
         }
@@ -867,7 +901,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         initializeConnectedTime();
         this.isLoggedIn = true;
         this.platform = null;
-        
+
         if (!hasBroker()) {
             //register the MBean
             getServerPlatform().registerMBean();
@@ -884,7 +918,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         if ((getDatasourcePlatform() instanceof DatabasePlatform) && getPlatform().getBatchWritingMechanism() != null) {
             getPlatform().getBatchWritingMechanism().initialize(this);
         }
-        
+
     }
 
     /**
@@ -896,7 +930,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             this.eventManager.postLogin(this);
         }
     }
-    
+
     /**
      * PUBLIC:
      * Connect to the database using the given user name and password.
@@ -904,6 +938,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * This is the login that should be used if each user has their own id,
      * but all users share the same database configuration.
      */
+    @Override
     public void login(String userName, String password) throws DatabaseException {
         getDatasourceLogin().setUserName(userName);
         getDatasourceLogin().setPassword(password);
@@ -916,6 +951,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * The login may also the preset and the login() protocol called.
      * This is the login should only be used if each user has their own database configuration.
      */
+    @Override
     public void login(Login login) throws DatabaseException {
         setLogin(login);
         login();
@@ -929,31 +965,32 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * @exception DatabaseException the database will also raise an error if their is an active transaction,
      * or a general error occurs.
      */
+    @Override
     public void logout() throws DatabaseException {
         if (this.eventManager != null) {
             this.eventManager.preLogout(this);
         }
-        
-        cleanUpEntityListenerInjectionManager();
-        
+
+        cleanUpInjectionManager();
+
         // Reset cached data, as may be invalid later on.
         this.lastDescriptorAccessed = null;
 
         if (isInTransaction()) {
             throw DatabaseException.logoutWhileTransactionInProgress();
         }
-        
+
         if (getAccessor() == null) {
             return;
         }
-        
+
         if (this.databaseEventListener != null) {
             this.databaseEventListener.remove(this);
         }
-        
+
         // We're logging out so turn off change propagation.
         setShouldPropagateChanges(false);
-        
+
         if (!hasBroker()) {
             if (getCommandManager() != null) {
                 getCommandManager().shutdown();
@@ -962,15 +999,15 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             // Unregister the JMX MBean before logout to avoid a javax.naming.NameNotFoundException
             getServerPlatform().shutdown();
         }
-        
+
         disconnect();
         getIdentityMapAccessor().initializeIdentityMaps();
         this.isLoggedIn = false;
         if (this.eventManager != null) {
             this.eventManager.postLogout(this);
         }
-        log(SessionLog.INFO, SessionLog.CONNECTION, "logout_successful", this.getName());
-	   
+        log(SessionLog.FINE, SessionLog.CONNECTION, "logout_successful", this.getName());
+
     }
 
     /**
@@ -1001,6 +1038,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
      * @exception OptimisticLockException if the object's descriptor is using optimistic locking and
      * the object has been updated or deleted by another user since it was last read.
      */
+    @Override
     public void writeAllObjects(Collection domainObjects) throws DatabaseException, OptimisticLockException {
         for (Iterator objectsEnum = domainObjects.iterator(); objectsEnum.hasNext();) {
             writeObject(objectsEnum.next());
@@ -1022,7 +1060,7 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
             writeObject(objectsEnum.nextElement());
         }
     }
-    
+
     /**
      * INTERNAL:
      * A query execution failed due to an invalid query.
@@ -1049,7 +1087,9 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
                 try {
                     // Give the failover time to recover.
                     Thread.currentThread().sleep(getLogin().getDelayBetweenConnectionAttempts());
-                    log(SessionLog.INFO, SessionLog.QUERY, "communication_failure_attempting_query_retry", (Object[])null, null);
+                    Object[] args = new Object[1];
+                    args[0] = ex;
+                    log(SessionLog.INFO, SessionLog.QUERY, "communication_failure_attempting_query_retry", args, null);
                 } catch (InterruptedException intEx) {
                     break;
                 }
@@ -1057,14 +1097,14 @@ public class DatabaseSessionImpl extends AbstractSession implements org.eclipse.
         }
         return executionSession.executeQuery(query, row, retryCount);
     }
-    
+
     /**
      * Return the tuner used to tune the configuration of this session.
      */
     public SessionTuner getTuner() {
         return tuner;
     }
-    
+
     /**
      * Set the tuner used to tune the configuration of this session.
      */
