@@ -1,15 +1,17 @@
-/*******************************************************************************
- * Copyright (c) 2011, 2013 Oracle. All rights reserved.
+/*
+ * Copyright (c) 2011, 2018 Oracle and/or its affiliates. All rights reserved.
+ *
  * This program and the accompanying materials are made available under the
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
- * which accompanies this distribution.
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     tware -  Initial implementation
- ******************************************************************************/
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     tware -  Initial implementation
 package org.eclipse.persistence.jpa.rs.util;
 
 import java.lang.reflect.Modifier;
@@ -47,7 +49,7 @@ import org.eclipse.persistence.sessions.SessionEvent;
 
 /**
  * This adapter alters the way the JAXBContext handles relationships for an existing persistence unit.
- * It changes non-private relationship mappings to be read-only and replaces those mappings with a mapping 
+ * It changes non-private relationship mappings to be read-only and replaces those mappings with a mapping
  * to a weaved-in list of relationships that will produce links.
  * @author tware
  *
@@ -68,6 +70,7 @@ public class PreLoginMappingAdapter extends SessionEventListener {
     /* (non-Javadoc)
      * @see org.eclipse.persistence.internal.jaxb.SessionEventListener#preLogin(org.eclipse.persistence.sessions.SessionEvent)
      */
+    @Override
     @SuppressWarnings({ "unchecked", "rawtypes" })
     public void preLogin(SessionEvent event) {
         Project project = event.getSession().getProject();
@@ -84,7 +87,7 @@ public class PreLoginMappingAdapter extends SessionEventListener {
                 XMLDescriptor xmlDescriptor = (XMLDescriptor) project.getAliasDescriptors().get(descriptorAlias);
                 if (null != xmlDescriptor) {
                     if (null == xmlDescriptor.getDefaultRootElement()) {
-                        // set root element 
+                        // set root element
                         xmlDescriptor.setDefaultRootElement(xmlNameTransformer.transformRootElementName(xmlDescriptor.getJavaClass().getName()));
                         // set resultAlwaysXMLRoot to false, so that the elements are not wrapped in JAXBElements when unmarshalling.
                         xmlDescriptor.setResultAlwaysXMLRoot(false);
@@ -147,14 +150,12 @@ public class PreLoginMappingAdapter extends SessionEventListener {
                             DatabaseMapping dbMapping = jpaDescriptor.getMappingForAttributeName(mapping.getAttributeName());
                             if ((dbMapping != null) && (dbMapping instanceof ForeignReferenceMapping)) {
                                 ForeignReferenceMapping jpaMapping = (ForeignReferenceMapping) dbMapping;
-                                if (jpaMapping != null) {
-                                    if (jpaMapping.getMappedBy() != null) {
-                                        ClassDescriptor inverseDescriptor = project.getDescriptorForAlias(jpaMapping.getReferenceDescriptor().getAlias());
-                                        if (inverseDescriptor != null) {
-                                            DatabaseMapping inverseMapping = inverseDescriptor.getMappingForAttributeName(jpaMapping.getMappedBy());
-                                            if (inverseMapping != null) {
-                                                convertMappingToXMLInverseReferenceMapping(inverseDescriptor, inverseMapping, jpaMapping);
-                                            }
+                                if (jpaMapping.getMappedBy() != null) {
+                                    ClassDescriptor inverseDescriptor = project.getDescriptorForAlias(jpaMapping.getReferenceDescriptor().getAlias());
+                                    if (inverseDescriptor != null) {
+                                        DatabaseMapping inverseMapping = inverseDescriptor.getMappingForAttributeName(jpaMapping.getMappedBy());
+                                        if (inverseMapping != null) {
+                                            convertMappingToXMLInverseReferenceMapping(inverseDescriptor, inverseMapping, jpaMapping);
                                         }
                                     }
                                 }
@@ -176,7 +177,7 @@ public class PreLoginMappingAdapter extends SessionEventListener {
                         if (value instanceof Class) {
                             subClassToInstantiate = (Class) value;
                             isAbstract = Modifier.isAbstract(subClassToInstantiate.getModifiers());
-                            if ((subClassToInstantiate != null) && (!isAbstract)) {
+                            if (!isAbstract) {
                                 InstantiationPolicy instantiationPolicy = new InstantiationPolicy();
                                 instantiationPolicy.useFactoryInstantiationPolicy(new ConcreteSubclassFactory(subClassToInstantiate), "createConcreteSubclass");
                                 descriptor.setInstantiationPolicy(instantiationPolicy);
@@ -200,27 +201,21 @@ public class PreLoginMappingAdapter extends SessionEventListener {
                             DatabaseMapping dbMapping = jpaDescriptor.getMappingForAttributeName(mapping.getAttributeName());
                             if ((dbMapping instanceof ForeignReferenceMapping)) {
                                 ForeignReferenceMapping jpaMapping = (ForeignReferenceMapping) dbMapping;
-                                if (jpaMapping != null) {
-                                    ClassDescriptor jaxbDescriptor = project.getDescriptorForAlias(jpaMapping.getDescriptor().getAlias());
-                                    if (jaxbDescriptor != null) {
-                                        convertMappingToXMLChoiceMapping(jaxbDescriptor, jpaMapping, cl, jpaSession);
-                                    }
-                                }
+                                ClassDescriptor jaxbDescriptor = project.getDescriptorForAlias(jpaMapping.getDescriptor().getAlias());
+                                convertMappingToXMLChoiceMapping(jaxbDescriptor, jpaMapping, cl, jpaSession);
                             }
                         } else if (mapping instanceof XMLCompositeObjectMapping) {
-                            // Fix for Bug 403113 - JPA-RS Isn't Serializing an Embeddable defined in an ElementCollection to JSON Correctly 
+                            // Fix for Bug 403113 - JPA-RS Isn't Serializing an Embeddable defined in an ElementCollection to JSON Correctly
                             // add choice mapping for one-to-one relationships within embeddables
-                            // Based on (http://wiki.eclipse.org/EclipseLink/Examples/JPA/NoSQL#Step_2_:_Map_the_data), 
+                            // Based on (http://wiki.eclipse.org/EclipseLink/Examples/JPA/NoSQL#Step_2_:_Map_the_data),
                             // the mappedBy option on relationships is not supported for NoSQL data, so no need to add inverse mapping
                             XMLCompositeObjectMapping jpaMapping = (XMLCompositeObjectMapping) mapping;
-                            if (jpaMapping != null) {
-                                ClassDescriptor jaxbDescriptor = project.getDescriptorForAlias(jpaMapping.getDescriptor().getAlias());
-                                if (jaxbDescriptor != null) {
-                                    Class clazz = jpaMapping.getReferenceClass();
-                                    if (clazz != null) {
-                                        if ((jpaSession.getDescriptor(clazz) != null) && (jpaSession.getDescriptor(clazz).isEISDescriptor()))
-                                            convertMappingToXMLChoiceMapping(jaxbDescriptor, jpaMapping, cl, jpaSession);
-                                    }
+                            ClassDescriptor jaxbDescriptor = project.getDescriptorForAlias(jpaMapping.getDescriptor().getAlias());
+                            if (jaxbDescriptor != null) {
+                                Class clazz = jpaMapping.getReferenceClass();
+                                if (clazz != null) {
+                                    if ((jpaSession.getDescriptor(clazz) != null) && (jpaSession.getDescriptor(clazz).isEISDescriptor()))
+                                        convertMappingToXMLChoiceMapping(jaxbDescriptor, jpaMapping, cl, jpaSession);
                                 }
                             }
                         }
@@ -300,7 +295,7 @@ public class PreLoginMappingAdapter extends SessionEventListener {
         if ((jpaMapping != null) && (jaxbDescriptor != null)) {
             if ((jpaMapping instanceof ForeignReferenceMapping) && ((jpaMapping.isAggregateCollectionMapping()) || (jpaMapping.isAggregateMapping()))) {
                 // Fix for Bug 402385 - JPA-RS: ClassNotFound when using ElementCollection of Embeddables
-                // Aggregates don't have identity to create links, thus no weaved REST adapters to insert choice mappings   
+                // Aggregates don't have identity to create links, thus no weaved REST adapters to insert choice mappings
                 return;
             }
 
