@@ -1,15 +1,18 @@
-/*******************************************************************************
- * Copyright (c) 1998, 2013 Oracle and/or its affiliates. All rights reserved.
- * This program and the accompanying materials are made available under the 
- * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
- * which accompanies this distribution. 
- * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
- * and the Eclipse Distribution License is available at 
+/*
+ * Copyright (c) 1998, 2021 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021 IBM Corporation. All rights reserved.
+ *
+ * This program and the accompanying materials are made available under the
+ * terms of the Eclipse Public License v. 2.0 which is available at
+ * http://www.eclipse.org/legal/epl-2.0,
+ * or the Eclipse Distribution License v. 1.0 which is available at
  * http://www.eclipse.org/org/documents/edl-v10.php.
  *
- * Contributors:
- *     Oracle - initial API and implementation from Oracle TopLink
- ******************************************************************************/  
+ * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
+ */
+
+// Contributors:
+//     Oracle - initial API and implementation from Oracle TopLink
 package org.eclipse.persistence.sessions;
 
 import java.util.*;
@@ -364,6 +367,26 @@ public class SessionEventManager extends CoreSessionEventManager<SessionEventLis
         endOperationProfile();
     }
 
+    /**
+     * INTERNAL:
+     * Post execute call.
+     */
+    public void postExecuteCall(Call call, Object result) {
+        if (!hasListeners()) {
+            return;
+        }
+        startOperationProfile();
+        SessionEvent event = new SessionEvent(SessionEvent.PostExecuteCall, getSession());
+        event.setCall(call);
+        event.setResult(result);
+        List<SessionEventListener> listeners = this.listeners;
+        int size = listeners.size();
+        for (int index = 0; index < size; index++) {
+            this.listeners.get(index).postExecuteCall(event);
+        }
+        endOperationProfile();
+    }
+
     /**
      * INTERNAL:
      * Post execute query.
@@ -585,6 +608,25 @@ public class SessionEventManager extends CoreSessionEventManager<SessionEventLis
         endOperationProfile();
     }
 
+    /**
+     * INTERNAL:
+     * Pre execute call.
+     */
+    public void preExecuteCall(Call call) {
+        if (!hasListeners()) {
+            return;
+        }
+        startOperationProfile();
+        SessionEvent event = new SessionEvent(SessionEvent.PreExecuteCall, getSession());
+        event.setCall(call);
+        List<SessionEventListener> listeners = this.listeners;
+        int size = listeners.size();
+        for (int index = 0; index < size; index++) {
+            listeners.get(index).preExecuteCall(event);
+        }
+        endOperationProfile();
+    }
+
     /**
      * INTERNAL:
      * Pre execute query.
@@ -836,7 +878,7 @@ public class SessionEventManager extends CoreSessionEventManager<SessionEventLis
                 synchronized(this) {
                     if (this.listeners != null) {
                         ArrayList<SessionEventListener> listenersCopy = new ArrayList(listeners);
-                        isRemoved = listenersCopy.remove(listener); 
+                        isRemoved = listenersCopy.remove(listener);
                         if (isRemoved) {
                             this.listeners = listenersCopy;
                         }
